#!/usr/bin/env python

'''

    @version $Id: aamaprotator,v 1.5 2005/10/19 05:52:58 davidfancella Exp $

    This file is part of aamaprotator.

    aamaprotator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    aamaprotator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with aamaprotator; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    aamaprotator is copyright 2005 by David Fancella

'''

import paths, os

class SettingsBase(object):
    filelist = None
    settings = None
    
    def __init__(self, filelist=None):
        self.filelist = []
        self.settings = []
        
        if isinstance(filelist, list):
            self.filelist = filelist

        self.LoadFiles(self.filelist)

    def GetSettings(self):
        return self.settings

    # set a setting.  It's used by the file parser, so subclasses should be able to
    # override it completely
    def Set(self, name, value):
        self.settings.append( (name, value) )

    # Get a setting.  This is pretty, um, inefficient.  Subclasses that expect to use
    # it a lot should override it
    def Get(self, name):
        value = ''
        for a in self.settings:
            if a[0] == name:
                value = a[1]

    def LoadFiles(self, filelist):
        for file in filelist:
            self._load_file(file)
            
    def _load_file(self, filename):
        head, tail = os.path.split(filename)
        fp = paths.OpenFile(filename, 'r')

        if fp is not None:
            for line in fp:
                theSetting = None
                theSetting = self.ParseLine(line)
                if theSetting is not None:
                    if theSetting[0] == 'INCLUDE':
                        self._load_file(os.path.join(head, theSetting[1]))
                    else:
                        self.Set(theSetting[0], theSetting[1])

            fp.close()

    def ParseLine(self, line):
        codeLine = line[0:line.find("#")]
        codeLine = codeLine.strip()
        setting = ''
        value = ''
        if len(codeLine) > 0:
            theSettingLine = codeLine.split(None, 1)
            
            if len(theSettingLine) > 0:
                setting = theSettingLine[0]
                if len(theSettingLine) == 2:
                    value = theSettingLine[1]
                else:
                    value = ''
            
            if (len(setting) > 0) and (len(value) > 0):
                return (setting.strip().upper(), value.strip())
            
        # No setting to be had here, return none    
        return None

# ArmaSettings takes a prefix to the settings directory containing your server's settings
# It will then load all of the files in the same order as the server to determine the
# default settings of the server
class ArmaSettings(SettingsBase):
    armaSettingFiles = [
                        'settings.cfg',
                        'settings_dedicated.cfg',
                        'autoexec.cfg'
                        ]

    defaultSettings = None
    
    def __init__(self, filename=None):
        tmpFilelist = []
        if filename is not None:
            for file in self.armaSettingFiles:
                tmpFilelist.append(os.path.join(filename, file))
            
        # now initialize the dictionary that contains the default settings
        # note: values that appear later in self.settings override values that appear
        # earlier
        self.defaultSettings = {}
        
        super(ArmaSettings, self).__init__(tmpFilelist)

        for setting in self.settings:
            self.defaultSettings[setting[0]] = setting[1]

    def Set(self, name, value):
        self.defaultSettings[name] = value

    def Get(self, name):
        if self.defaultSettings.has_key(name):
            return self.defaultSettings[name]

        return ''

    def GetSettings(self):
        retList = []
        for a in self.defaultSettings.keys():
            retList.append( (a, self.defaultSettings[a]) )

        return retList

    # returns the default settings for each setting contained in the SettingsBase child
    # passed in settingsObj
    def GetDefaultDiff(self, settingsObj):
        itsSettings = settingsObj.GetSettings()
        retSettings = []
        for item in itsSettings:
            if self.defaultSettings.has_key(item[0]):
                retSettings.append( (item[0], self.defaultSettings[item[0]]) )

        return retSettings

# takes a single string to a file located in aamaprotator's settings directory
class aaMapSettings(SettingsBase):
    mySettings = None

    def __init__(self, filename=None):
        # now initialize the dictionary that contains the settings
        # note: values that appear later in self.settings override values that appear
        # earlier
        # sorry for the funky naming, I copied and pasted it from the other class
        # hurray for shitty code reuse
        self.mySettings = {}

        super(aaMapSettings, self).__init__(filename)
    
        for setting in self.settings:
            self.mySettings[setting[0]] = setting[1]

    def LoadFiles(self, filelist):
        for file in filelist:
            self._load_file(os.path.join(paths.GetSettingsDir(), file))
    
    def Set(self, name, value):
        self.mySettings[name] = value

    def Get(self, name):
        if self.mySettings.has_key(name):
            return self.mySettings[name]

        return ''

    def GetSettings(self):
        retList = []
        for a in self.mySettings.keys():
            retList.append( (a, self.mySettings[a]) )

        return retList


